/*
 * Copyright (c) 2009, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <math.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include <unistd.h>
#include <termios.h>


int main(void) 
{
	struct termios oldterm, term;
	struct sockaddr_in sin_ci;
	fd_set fds;
	int s;
	char c;

	s = socket(PF_INET, SOCK_STREAM, 0);
	if (s < 0) {
		printf("socket() failed\n");
		return s;
	}

	/* XXX specify address */
	memset(&sin_ci, 0, sizeof(sin_ci));
	sin_ci.sin_family = AF_INET;
	inet_pton(AF_INET, "127.0.0.1", &sin_ci.sin_addr.s_addr);
	sin_ci.sin_port = htons(1234);
#ifndef __linux__
	sin_ci.sin_len = sizeof(sin_ci);
#endif

	if (connect(s, (struct sockaddr *)&sin_ci, sizeof(sin_ci)) < 0) {
		printf("connect() failed\n");
		return s;
	}

	tcgetattr(0, &oldterm);
	memcpy(&term, &oldterm, sizeof(term));
	term.c_lflag &= ~(ICANON | ECHO | ISIG);
	tcsetattr(0, TCSANOW, &term);
	setbuf(stdin, NULL);

	while (1) {
		FD_ZERO(&fds);
		FD_SET(0, &fds);
		FD_SET(s, &fds);
		if (select(s+1, &fds, NULL, NULL, NULL) < 0)
			break;
		if (FD_ISSET(0, &fds)) {
			if (read(0, &c, 1) <= 0)
				break;
			write(s, &c, 1);
		}
		if (FD_ISSET(s, &fds)) {
			if (read(s, &c, 1) <= 0)
				break;
			write(1, &c, 1);
		}
	}
	tcsetattr(0, TCSANOW, &oldterm);
	printf("\n");

	return 0;
}


